//Program to create inital pool of variants in a crossing experiment


#include <iostream>
#include <sstream>
#include <vector>
#include <list>
#include <deque>

using namespace std;
#include "crossing-tools.h"

void write_vector(std::string name, std::vector <double> &vec);

int main(int argc, char *argv[]){
	
	
	if(argc != 9){
		cout<< "Usage ./run_crossing N n_loci dist_between_loci rho Ngen Nsample seed output_dir" <<endl;
		exit(0);
	}
	
	int N=(int)atof(argv[1]);/* how large population */
	int n_loci=atoi(argv[2]);/* how many segregating sites */
	int dist=atoi(argv[3]);/* distance between segregating sites */
	double rho=atof(argv[4]);/* base rate of recombination per generation per basepair */
	int Ngen=atoi(argv[5]);/* number of generations of crossing */
	int Nsample=atoi(argv[6]);/* number of haplotypes sampled from the final pool */
	int seed=atoi(argv[7]); /* seed for random number generator */
	string outdir(argv[8]); /* output directory for saving the used recombination profile */
	
	gsl_rng_env_setup();
	srand((unsigned int)(seed)); 
	
	gsl_rng *rgen = gsl_rng_alloc (gsl_rng_taus);
	gsl_rng_set (rgen, seed);
	
	vector < std::string > population;
	vector < std::string  > populationNew;

	init_population(population,N,n_loci);
	populationNew=population;
	
	double nlRho=(double)N*(double)dist*(double)n_loci*rho;
	
	vector <double> nlRhoV(n_loci-1);
	vector <double> rhoV(n_loci-1);
	
	for(int i=0; i< nlRhoV.size(); i++){
		
		/* making two hotspot regions with 50-fold higher rate */ 
		if(i > n_loci/4 && i< 2*n_loci/4 || i > 3*n_loci/4){
			nlRhoV[i]=50.0*nlRho/(double)n_loci;
		}
		else{
			nlRhoV[i]=nlRho/(double)n_loci;
		}

	}
		
	for(int iC=0; iC< Ngen; iC++){
		combined_resample_cross_rhoV(population, populationNew, nlRhoV, rgen);
		population=populationNew;				
	}
	print_population_sample(Nsample, population);
	
	for(int i=0; i< nlRhoV.size(); i++){
		rhoV[i]=nlRhoV[i]/(double)N;
	}
	
	write_vector(outdir+"/simulatedRhoVec.txt", rhoV);
	
	
	
	
	return EXIT_SUCCESS;
}



void write_vector(std::string name, std::vector <double> &vec){
	ofstream file;
	file.open(name.c_str());
	
	for(int i=0; i<vec.size(); i++){
		file << vec[i] << endl;
	}
	
	
	file.close();
}




							
					
 
							
	
