

#include "crossing-tools.h"


/* find on which interval a random number falls on a cumulative probability vector */
int pickFather(vector <double> &cprob){
	
	double rn=(double)(rand())/(double)(RAND_MAX);
	int k=0;
	while(k<cprob.size()){
		if(rn<cprob[k]){
			return k;
		}
		k++;
	}
	return -1; /* this should not happen*/  
}

/* this needs to be modified if more than two founder strains or unequal numbers of two parental strains form the inital pool */

void init_population(vector < std::string > &population, int N, int n_loci){
	
	std::string ones;
	std::string zeros;
	
	for(int l=0;l<n_loci; l++){
		ones.append("1");
		zeros.append("0");
	}

	for(int n=0;n<N/2; n++){
		population.push_back(ones);
		population.push_back(zeros);
	}
}


/* print a random sample of haplotypes (with replacement)  in a format which can be read by LDhat */
void print_population_sample(int sample, vector < std::string > &population){
	
	int N=population.size();
	int n_loci=population[0].size();
	
	cout << sample <<" "<< n_loci << " 1" << endl;
	
	
	for(int n=0; n<sample; n++){
	
		double rn=(double)(rand())/(double)(RAND_MAX);
		
		int ri=(int)(rn*(double)(N));
		
		cout << ">seq"<<n<<"\n";
		cout << population[ri] << endl;

	}
	
	
}


/* draw new generation with replacement and recombine with recombination profile rhoV */

void combined_resample_cross_rhoV(vector < std::string > &population, vector < std::string > &populationNew, vector <double> &nlRho, gsl_rng *gBaseRand){
	
	
	int N=population.size();
	int n_loci=population[0].size();
	
	std::string recombinant1;
	std::string recombinant2;
	std::string recombinant1N;
	std::string recombinant2N;
	
	int k=0;
	
	double nlRhoZ=0.0;
	
	for(int i=0; i< nlRho.size(); i++){
		nlRhoZ+=nlRho[i];
	}
	
	vector <double> cumProb(nlRho.size());
	
	
	cumProb[0]=nlRho[0];
	for(int i=1; i< nlRho.size(); i++){
		cumProb[i]=nlRho[i]+cumProb[i-1];
	}
	
	for(int i=0; i< nlRho.size(); i++){
		cumProb[i]=cumProb[i]/nlRhoZ;
		
	}
	
	for(int n=0; n< N/2; n++){
		
		double rn=(double)(rand())/(double)(RAND_MAX);
		int ind1=(int)(rn*(double)(N));
		
		rn=(double)(rand())/(double)(RAND_MAX);
		int ind2=(int)(rn*(double)(N));
		
		
		recombinant1=population[ind1];
		recombinant2=population[ind2];
		
		recombinant1N=recombinant1;
		recombinant2N=recombinant2;
	
		/* this many events to be distributed */
		
		int nEvents=(int)gsl_ran_poisson(gBaseRand,nlRhoZ/(double)N);
		
		vector <int> breakP;
		
		breakP.clear();
		
		for(int ri=0; ri<nEvents; ri++){
			/* get a random break point */			
			double rn=(double)(rand())/(double)(RAND_MAX);
			int rx=pickFather(cumProb);
			breakP.push_back(rx+1);
		}
		breakP.push_back(n_loci);
		
		
		recombinant1N=form_recombinant(breakP,recombinant1,recombinant2);
		recombinant2N=form_recombinant(breakP,recombinant2,recombinant1);
		
		populationNew[k++]=recombinant1N;
		populationNew[k++]=recombinant2N;
		
		
	}
	
}

string form_recombinant(vector <int> &breakP, string seq1, string seq2){

	string recombinant;
	recombinant=seq1;
	
	sort(breakP.begin(), breakP.end());
	for(int ri=1; ri<breakP.size(); ri=ri+2){
		for(int l=breakP[ri-1]; l < breakP[ri]; l++){
			recombinant[l]=seq2[l];
		}
	}	
	return recombinant;
	
};








