//Program to create calculate likelihood surface for large "infinite" population in a crossing experiment.

#include "utilities.h"
using namespace std;

void get_qvector_general(double x0, double y0, int Ncrosses,  double rho, vector < double> &q);

int main(int argc, char *argv[]){
	
	
	
	if(argc != 7){
		cout<< "Usage ./run_infinite Ngrid rMax Ncrosses Nsample q1init_i q1init_j" <<endl;
		exit(0);
	}

	int Ngrid=(int)atof(argv[1]);/* number of points in a rho grid */
	double rMax=atof(argv[2]); /* maximum rho considered */
	int Ncrosses=(int)atof(argv[3]);/* number of crossing generations */
	int Nsample=(int)atof(argv[4]);/* number of haplotypes drawn from the final pool */	
	double x0Init=atof(argv[5]);  /* allele frequency initial condition at locus i */
	double y0Init=atof(argv[6]);  /* allele frequency initial condition at locus j */
	

	vector < double> rhoV;
	for(int i=0; i< Ngrid; i++){
		double rho=i*rMax/(double)(Ngrid-1);
		rhoV.push_back(rho);
	}
	
	vector < observation > table;
	create_pairs(Nsample, table);
	

	int Npairs=table.size();
	
	vector <double> q;
	vector <double> ll;
	
	
	print_header(Nsample,Npairs,Ngrid,rhoV);
	
	for(int i=0; i< table.size(); i++){	
		ll.resize(0);
		for(int ri=0; ri<rhoV.size(); ri++){
			get_qvector_general(x0Init,y0Init,Ncrosses,rhoV[ri],q);
			double val=log_likehood(table[i].draw,q);
			ll.push_back(val);
		}
		print_line(i+1, table[i].draw, ll);
	};
	
	return EXIT_SUCCESS;
};

/* Infinite population size approximation for the two locus genotype frequencing undergoing recombination for Ncrosses generations */
void get_qvector_general(double x0, double y0, int Ncrosses,  double rho, vector < double> &q){
	
	double eps=0.001;
	double x1,y1;
	
	x1=1.0-x0;
	y1=1.0-y0;
	
	double dijmax=min(x0*y1,x1*y0);
	double dij;
	
	/* rho=1/2 equals free recombination */
	dij = dijmax*pow((1.0 - min(0.5, rho)), (double)Ncrosses);
	
	q.clear();
	q.push_back(y0*x0 + dij); // q00;
	q.push_back(x0*y1 - dij); // q01;
	q.push_back(x1*y0 - dij); // q10;
	q.push_back(x1*y1 + dij); // q11;
	
	/* for zero entries add epsilon and renormalize accordingly to avoid -infinities in the log likehood for zero recombination case */
	for(int i=0; i< q.size(); i++){
		if(q[i]<eps){
			q[i]=eps;
		}
	}
	double z=0.0;
	for(int i=0; i< q.size(); i++){
		z+=q[i];
	}
	for(int i=0; i< q.size(); i++){
		q[i]=q[i]/z;
	}
	
};



							
					
 
							
	
